/////////////////////////////////////////////////////////////
// This source code is Copyright (C) David Wilkinson 1996.
// All rights reserved.
//
// For more information,
//
//    Email: davidw@cascade.demon.co.uk
//    URL:   http://www.netlink.co.uk/users/cascade/http/
//
/////////////////////////////////////////////////////////////


// $Id: HTTPConfiguration.java,v 1.5 1996/08/04 15:22:33 davidw Exp $
//
// $Log: HTTPConfiguration.java,v $
// Revision 1.5  1996/08/04 15:22:33  davidw
// Added copyright
//
// Revision 1.4  1996/06/23 12:36:05  davidw
// Support for Include directive
//
// Revision 1.3  1996/05/06 15:37:25  davidw
// Made into package uk.co.demon.cascade.http
//
// Revision 1.2  1996/04/28 14:03:04  davidw
// Made error messages more verbose
//
// Revision 1.1  1996/04/13 22:04:30  davidw
// Initial revision
//

package uk.co.demon.cascade.http;
import uk.co.demon.cascade.http.*;


import java.lang.*;
import java.io.*;
import java.util.*;


/**
 * HTTPConfiguration class. 
 *
 * @version 	$Revision: 1.5 $, $Date: 1996/08/04 15:22:33 $
 *
 * @author David Wilkinson
 */


public class HTTPConfiguration
{
    private StreamTokenizer tokenizer;
    private Vector classList;

    private static Hashtable config;

    static
    {
	config = new Hashtable();
    }


    /** Open a new configuration file and parse it 
     *   @param fileName	the file to open
    */
    public HTTPConfiguration(String fileName)
    {
	try
	{
	    classList = new Vector();

	    HTTPSystem.verbose("Parsing config file " + fileName);

	    tokenizer = new StreamTokenizer(new FileInputStream(fileName));
	    tokenizer.resetSyntax();
	    tokenizer.whitespaceChars(0, 32);
	    tokenizer.wordChars(33, 126);
	    tokenizer.whitespaceChars(127, 255);
	    tokenizer.commentChar('#');
	    tokenizer.quoteChar('"');
	    tokenizer.quoteChar('\'');
	    tokenizer.eolIsSignificant(true);
	    tokenizer.slashSlashComments(false);
	    tokenizer.slashStarComments(true);

	    parseFile();
	}
	catch (FileNotFoundException e)
	{
	    HTTPSystem.fatalError("Cannot open configuration file " 
				  + fileName + ": " + e.getMessage());
	}
    }


    private void parseFile()
    {
	try
	{	
	    tokenizer.nextToken();

	    while (tokenizer.ttype != tokenizer.TT_EOF)
	    {
		// A line at a time
		Vector tokens = new Vector();
		
		while (tokenizer.ttype != tokenizer.TT_EOL &&
		       tokenizer.ttype != tokenizer.TT_EOF)
		{
		    if (tokenizer.ttype == tokenizer.TT_WORD)
		    {
			HTTPSystem.debug(3, "TT_WORD: " + tokenizer.sval);
			tokens.addElement(tokenizer.sval);
		    }

		    if (tokenizer.ttype == tokenizer.TT_NUMBER)
		    {
			HTTPSystem.debug(3, "TT_NUMBER: " + Double.toString(tokenizer.nval));
			tokens.addElement(Double.toString(tokenizer.nval));
		    }

		    tokenizer.nextToken();
		}

		// We have a lineful of tokens, let's deal with them
	    
		configure(tokens);
		
		tokenizer.nextToken();
	    }
	}
	catch (IOException e)
	{
	    HTTPSystem.fatalError("IOException processing config file: " + e.getMessage());
	}
    }


    private void configure(Vector tokens)
    {
	if (tokens.size() > 0 && tokens.elementAt(0) != null)
	{
	    HTTPSystem.debug("Processing token: " + tokens.elementAt(0));
	    
	    String key =(String)tokens.elementAt(0) ;

	    String classname = Info.configurationClassPrefix + key;
	
	    HTTPSystem.debug("Loading class " + classname);

	    try
	    {
		Class configClass = Class.forName(classname);
		Class interfaces[] = configClass.getInterfaces();
	    
		if (isConfigurationClass(interfaces))
		{
		    ConfigurationClass obj = (ConfigurationClass)configClass.newInstance();
		    if (obj.configure(tokens))
		    {
			// Returned true, need to store away
			classList.addElement(obj);
		    }
		}
		else
		{
		    HTTPSystem.fatalError("Invalid configuration class: "
					  + classname);
		}
	    }
	    catch (ClassNotFoundException e)
	    {
		HTTPSystem.fatalError("Unknown configuration file directive: "
				      + tokens.elementAt(0));
	    }
	    catch (InstantiationException e)
	    {
		HTTPSystem.fatalError("Could not instantiate configuration class "
				      + classname + ": " + e.getMessage());
	    }
	    catch (IllegalAccessException e)
	    {
		HTTPSystem.fatalError("Could not initialise configuration class "
				      + classname + ": " + e.getMessage());
	    }
	    catch (InvalidHTTPConfigurationException e)
	    {
		HTTPSystem.fatalError("Error in configuration file for directive "
				      + (String)tokens.elementAt(0) + ": " + e.getMessage());
	    }
	}
    }


    private boolean isConfigurationClass(Class interfaces[])
    {
	boolean result = false;
	int i = 0;

	try
	{
	    while (true)
	    {
		Class c = interfaces[i];

		if (c.getName().equals(Info.configurationClassInterface))
		{
		    result = true;
		    break;
		}
		i++;
	    }
	}
	catch (ArrayIndexOutOfBoundsException e)
	{
	    // Gone too far, just stop
	}

	return result;
    }


    public static void put(String key, String value)
    {
	synchronized (config)
	{
	    config.put(key, value);
	}
    }
  

    public static String get(String key)
    {
	String value;

	synchronized (config)
	{
	    value = (String) config.get(key);
	}
	return value;
    }
}

