/////////////////////////////////////////////////////////////
// This source code is Copyright (C) David Wilkinson 1996.
// All rights reserved.
//
// For more information,
//
//    Email: davidw@cascade.demon.co.uk
//    URL:   http://www.netlink.co.uk/users/cascade/http/
//
/////////////////////////////////////////////////////////////

// A class used to retrieve an object for the server

// $Id: HTTPServerExtensionObject.java,v 1.6 1996/08/04 15:22:33 davidw Exp $
//
// $Log: HTTPServerExtensionObject.java,v $
// Revision 1.6  1996/08/04 15:22:33  davidw
// Added copyright
//
// Revision 1.5  1996/06/23 12:55:12  davidw
// Use HTTPInformation instead of HTTPEnvironment
//
// Revision 1.4  1996/06/03 20:16:08  davidw
// Use HTTPEnvironment
//
// Revision 1.3  1996/05/06 15:37:25  davidw
// Made into package uk.co.demon.cascade.http
//
// Revision 1.2  1996/04/28 14:04:27  davidw
// Major rewrite
//
// Revision 1.1  1996/04/28 09:54:13  davidw
// Initial revision
//


package uk.co.demon.cascade.http;
import uk.co.demon.cascade.http.*;


import java.lang.*;
import java.io.*;
import java.util.*;


class HTTPServerExtensionObject extends HTTPObject
{
    private HTTPServerExtension obj;
    private Thread thread;

    private PipedInputStream  pis;
    private PipedOutputStream pos;

    public HTTPServerExtensionObject(String path, HTTPInformation i) throws HTTPException
    {
	super(i);

	HTTPSystem.debug("Loading class " + path);
	
	setHeaderGeneration(true);

	try
	{
	    Class extClass = Class.forName(path);
	    Class interfaces[] = extClass.getInterfaces();
	    
	    if (isExtensionClass(interfaces))
	    {
		HTTPServerExtension extObj = (HTTPServerExtension)extClass.newInstance();
		objectIs = new PipedInputStream();
		objectOs = new PipedOutputStream();
		pis = new PipedInputStream();
		pos = new PipedOutputStream();

		((PipedInputStream)objectIs).connect(pos);
		pos.connect((PipedInputStream)objectIs);

		pis.connect((PipedOutputStream)objectOs);
		((PipedOutputStream)objectOs).connect(pis);


		extObj.init(info, pis, pos);

		thread = new Thread(extObj);
		thread.start();
		    
	    }
	    else
	    {
		HTTPException h = new HTTPException("Invalid Extension class: " + path);
		h.setStatus(HTTPStatus.INTERNAL_ERROR);
		throw h;
	    }
	}
	catch (ClassNotFoundException e)
	{
	    HTTPException h = new HTTPException("Extension class "
						+ path
						+ " not found");
	    h.setStatus(HTTPStatus.INTERNAL_ERROR);
	    throw h;
	}
	catch (InstantiationException e)
	{
	    HTTPException h = new HTTPException("Could not instantiate Extension class " + path);
	    throw h;
	}
	catch (IllegalAccessException e)
	{
	    HTTPException h = new HTTPException("Could not initialise exception class "
						+ path + ": " 
						+ e.getMessage());
	    throw h;
	}
	catch (IOException e)
	{
	    HTTPException h = new HTTPException("IOException creating pipes to extension class "
						+ path + ": " 
						+ e.getMessage());
	    throw h;
	}
    }


    public void retrieve()
    {
	// Just wait for the object to finish & the pipes to do their stuff
	
	boolean done = false;

	while (!done)
	{
	    // Let everyone else have a go while we wait
	    yield();

	    done = true;

	    if (pipeIn != null && pipeIn.stillAlive())
	    {
		done = false;
	    }

	    if (pipeOut != null && pipeOut.stillAlive())
	    {
		done = false;
	    }

	    if (thread.isAlive())
	    {
		done = false;
	    }
	}

	try
	{
	    objectIs.close();
	    objectOs.close();
	    pis.close();
	    pos.close();
	}
	catch (IOException e)
	{
	    HTTPSystem.debug("IOException closing server extension streams: " +
			     e.getMessage());
	}
    }


    private boolean isExtensionClass(Class interfaces[])
    {
	boolean result = false;
	int i = 0;

	try
	{
	    while (true)
	    {
		Class c = interfaces[i];

		if (c.getName().equals(Info.extensionClassInterface))
		{
		    result = true;
		    break;
		}
		i++;
	    }
	}
	catch (ArrayIndexOutOfBoundsException e)
	{
	    // Gone too far, just stop
	}

	return result;
    }
}

