/////////////////////////////////////////////////////////////
// This source code is Copyright (C) David Wilkinson 1996.
// All rights reserved.
//
// For more information,
//
//    Email: davidw@cascade.demon.co.uk
//    URL:   http://www.netlink.co.uk/users/cascade/http/
//
/////////////////////////////////////////////////////////////


// $Id: TranslationObject.java,v 1.3 1996/08/04 15:22:33 davidw Exp $
//
// $Log: TranslationObject.java,v $
// Revision 1.3  1996/08/04 15:22:33  davidw
// Added copyright
//
// Revision 1.2  1996/05/06 15:37:25  davidw
// Made into package uk.co.demon.cascade.http
//
// Revision 1.1  1996/04/28 09:55:31  davidw
// Initial revision
//

package uk.co.demon.cascade.http;
import uk.co.demon.cascade.http.*;


import java.io.*;
import java.util.*;

public class TranslationObject extends Object
{
    private String from;
    private String to;
    private int type;

    public final static int MAP        = 1;
    public final static int PASS       = 2;
    public final static int EXEC       = 3;
    public final static int CLASSEXEC  = 4;

    public TranslationObject(int objType, String f, String t)
    {
	type = objType;
	from = f;
	to   = t;
    }
    
    public int getType()
    {
	return type;
    }

    public String translate(String s)
    {
	// Return translated if success, else return null

	int index = from.indexOf('*');

	if (index == -1)
	{
	    // Easy, no wildcards, just check equality

	    if (from.equals(s))
	    {
		return to;
	    }
	    else
	    {
		return null;
	    }
	}

	// A bit harder, need to process the wildcard
	// Only one wildcard (*) allowed per entry

	try
	{
	    String a = null;
	    String b = null;
	    
	    if (index > 0)
	    {
		a = from.substring(0, index);
	    }

	    if (index < from.length() - 1)
	    {
		b = from.substring(index + 1);
	    }

	    boolean matches = true;

	    if (a != null && !s.startsWith(a))
	    {
		matches = false;
	    }

	    if (b != null && !s.endsWith(b))
	    {
		matches = false;
	    }

	    if (matches)
	    {
		int aLength = 0;
		int bLength = 0;
		
		//if (a != null)
		//{
		//    aLength = a.length();
		//}
		
		if (b != null)
		{
		    bLength = b.length();
		}

		String wild = s.substring(index, s.length() - bLength);

		return replaceWildcard(wild);
	    }
	    
	}
	catch (StringIndexOutOfBoundsException e)
	{
	    return null;
	}
	
	return null;
    }


    private String replaceWildcard(String wild)
    {
	StringBuffer buff = new StringBuffer();

	String a = "";
	String b = "";
	
	int index = to.indexOf('*');

	if (index == -1)
	{
	    // Easy, no wildcards
	    return to;
	}
	    
	if (index > 0)
	{
	    a = to.substring(0, index);
	}

	if (index < from.length() - 1)
	{
	    b = to.substring(index + 1);
	}

	return a + wild + b;
    }
}

